/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.system.user.service.impl;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.AllArgsConstructor;
import org.springblade.core.mp.base.BaseEntity;
import org.springblade.core.mp.base.BaseServiceImpl;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tenant.mp.TenantEntity;
import org.springblade.system.user.entity.User;
import org.springblade.system.user.mapper.UserMapper;
import org.springblade.system.user.service.IUserSearchService;
import org.springframework.stereotype.Service;

import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 用户查询服务实现类
 *
 * @author Chill
 */
@Service
@AllArgsConstructor
public class UserSearchServiceImpl extends BaseServiceImpl<UserMapper, User> implements IUserSearchService {

    @Override
    public List<User> listByUser(List<Long> userId) {
        return this.list(Wrappers.<User>lambdaQuery().in(User::getId, userId));
    }

    @Override
    public List<User> listByDept(List<Long> deptId) {
        LambdaQueryWrapper<User> queryWrapper = Wrappers.lambdaQuery();
        deptId.forEach(id -> queryWrapper.like(User::getDeptId, id).or());
        return this.list(queryWrapper);
    }

    @Override
    public List<User> listByPost(List<Long> postId) {
        LambdaQueryWrapper<User> queryWrapper = Wrappers.lambdaQuery();
        postId.forEach(id -> queryWrapper.like(User::getPostId, id).or());
        return this.list(queryWrapper);
    }

    @Override
    public List<User> listByRole(List<Long> roleId) {
        LambdaQueryWrapper<User> queryWrapper = Wrappers.lambdaQuery();
        roleId.forEach(id -> queryWrapper.like(User::getRoleId, id).or());
        return this.list(queryWrapper);
    }

    @Override
    public Map<Long, String> mapByUser(Collection<Long> idList) {
        if (idList == null || idList.size() == 0) {
            return new HashMap<>();
        }

        List<User> list = lambdaQuery().select(BaseEntity::getId, User::getName).in(User::getId, idList).list();
        return list.stream().collect(Collectors.toMap(User::getId, User::getName));
    }

    @Override
    public Map<Long, Map<String, Object>> getUserNameAndPostByUserId(List<Long> userIdList) {
        List<Map<String, Object>> list = baseMapper.getUserNameAndPostByUserId(
                Wrappers.query().in("bu.id", userIdList).groupBy("bu.id"));

        return list.stream().collect(Collectors.toMap(
                item -> (Long) item.get("userId"),
                item -> item
        ));
    }

    @Override
    public Map<String, Long> getUserIdByJobNo(List<String> jobNoList) {
        List<User> list = list(Wrappers.<User>lambdaQuery()
                .select(BaseEntity::getId, User::getJobNo)
                .in(User::getJobNo, jobNoList)
                .eq(TenantEntity::getTenantId, AuthUtil.getTenantId())
        );

        return list.stream().collect(Collectors.toMap(
                User::getJobNo,
                BaseEntity::getId,
                (v1, v2) -> v2
        ));
    }

}
