/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package com.fjhx.entity.scheduling;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.math.BigDecimal;
import java.util.Date;

/**
 * 仓库实体类
 *
 * @author BladeX
 * @since 2022-07-27
 */
@Data
@TableName("stock_scheduling")
@ApiModel(value = "Scheduling对象", description = "仓库")
public class Scheduling {

    private static final long serialVersionUID = 1L;

    /**
     * 主键ID
     */
    @ApiModelProperty(value = "主键ID")
    @TableId(value = "id", type = IdType.ASSIGN_ID)
    private String id;
    /**
     * 是否删除
     */
    @ApiModelProperty(value = "是否删除")
    private Integer isDelete;
    /**
     * 排班日期
     */
    @ApiModelProperty(value = "排班日期")
    private Date planDate;
    /**
     * 喷印人员工号
     */
    @ApiModelProperty(value = "喷印人员工号")
    private String printer;
    /**
     * 打纸人员工号
     */
    @ApiModelProperty(value = "打纸人员工号")
    private String paperMan;
    /**
     * 物料编码
     */
    @ApiModelProperty(value = "物料编码")
    private String materialCode;
    /**
     * 物料长度
     */
    @ApiModelProperty(value = "物料长度")
    private BigDecimal materialQuantity;
    /**
     * 转印纸编码
     */
    @ApiModelProperty(value = "转印纸编码")
    private String paperCode;
    /**
     * 转印纸长度
     */
    @ApiModelProperty(value = "转印纸长度")
    private BigDecimal paperQuantity;
    /**
     * 创建时间
     */
    @ApiModelProperty(value = "创建时间")
    private Date createTime;
    /**
     * 修改时间
     */
    @ApiModelProperty(value = "修改时间")
    private Date updateTime;
    /**
     * 创建人
     */
    @ApiModelProperty(value = "创建人")
    private String createUser;
    /**
     * 修改人
     */
    @ApiModelProperty(value = "修改人")
    private String updateUser;

    /**
     * 喷印人员领料状态（0 未领取 1已领取）
     */
    private Integer materialStatus;

    /**
     * 打纸人员领料状态（0 未领取 1已领取）
     */
    private Integer paperStatus;

    /**
     * 喷印人员出入库单号
     */
    private Long materialFlag;

    /**
     * 打纸人员出入库单号
     */
    private Long paperFlag;

}
