package com.fjhx.enums.classify;

import org.springblade.core.tool.utils.Func;
import org.springblade.core.tool.utils.StringPool;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 * 分类类型枚举
 */
public enum ClassifyTypeEnum {
    CLASSIFY_TYPE_1(1, "产品分类"),
    CLASSIFY_TYPE_2(2, "物料分类"),
    ;

    private int key;

    private String value;

    private static Map<Integer, String> map = new LinkedHashMap<>();

    private static final HashMap<Integer, ClassifyTypeEnum> classMap = new HashMap<>();

    ClassifyTypeEnum(int key, String value) {
        this.key = key;
        this.value = value;
    }


    static {
        for (ClassifyTypeEnum value : ClassifyTypeEnum.values()) {
            classMap.put(value.getKey(), value);
        }
    }

    /**
     * 根据type获取枚举
     */
    public static ClassifyTypeEnum get(Integer type) {
        return classMap.get(type);
    }

    /**
     * 获取枚举map
     *
     * @return
     */
    public static Map<Integer, String> getMap() {
        if (Func.isNotEmpty(map)) {
            return map;
        }
        for (ClassifyTypeEnum ms : values()) {
            map.put(ms.key, ms.value);
        }
        return map;
    }

    /**
     * 通过key获取名称
     *
     * @param key
     * @return
     */
    public static String getNameByKey(Integer key) {
        if (key == null || key < 0) {
            return StringPool.EMPTY;
        }
        Map<Integer, String> map = getMap();
        return map.getOrDefault(key, StringPool.EMPTY);
    }

    public int getKey() {
        return key;
    }

    public String getValue() {
        return value;
    }

}
