package com.fjhx.enums.purchase;

import org.springblade.core.tool.utils.Func;
import org.springblade.core.tool.utils.StringPool;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 * 采购状态枚举
 */
public enum PurchaseStatusEnum {
    STATUS_10(10, "审批中"),
    STATUS_20(20, "驳回"),
    STATUS_30(30, "待发货"),
    STATUS_40(40, "已发货"),
    STATUS_50(50, "已完成"),
    ;

    private int key;

    private String value;

    private static Map<Integer, String> map = new LinkedHashMap<>();

    private static final HashMap<Integer, PurchaseStatusEnum> classMap = new HashMap<>();

    PurchaseStatusEnum(int key, String value) {
        this.key = key;
        this.value = value;
    }


    static {
        for (PurchaseStatusEnum value : PurchaseStatusEnum.values()) {
            classMap.put(value.getKey(), value);
        }
    }

    /**
     * 根据type获取枚举
     */
    public static PurchaseStatusEnum get(Integer type) {
        return classMap.get(type);
    }

    /**
     * 获取枚举map
     *
     * @return
     */
    public static Map<Integer, String> getMap() {
        if (Func.isNotEmpty(map)) {
            return map;
        }
        for (PurchaseStatusEnum ms : values()) {
            map.put(ms.key, ms.value);
        }
        return map;
    }

    /**
     * 通过key获取名称
     *
     * @param key
     * @return
     */
    public static String getNameByKey(Integer key) {
        if (key == null || key < 0) {
            return StringPool.EMPTY;
        }
        Map<Integer, String> map = getMap();
        return map.getOrDefault(key, StringPool.EMPTY);
    }

    public int getKey() {
        return key;
    }

    public String getValue() {
        return value;
    }

}
