package com.fjhx.enums.stock;

import cn.hutool.core.util.ObjectUtil;
import lombok.Getter;
import org.springblade.core.tool.utils.StringPool;

import java.util.HashMap;
import java.util.Map;

/**
 * 入库类型枚举
 */
@Getter
public enum InTypeEnum {
    PURCHASE(1, "采购入库"),
    MANUAL(2, "手动入库"),
    ;

    private final int key;

    private final String value;

    InTypeEnum(int key, String value) {
        this.key = key;
        this.value = value;
    }

    private static final Map<Integer, InTypeEnum> classMap = new HashMap<>();

    static {
        for (InTypeEnum value : InTypeEnum.values()) {
            classMap.put(value.getKey(), value);
        }
    }

    /**
     * 根据key获取枚举
     */
    public static InTypeEnum getEnumByKey(Integer type) {
        return classMap.get(type);
    }

    /**
     * 通过key获取名称
     */
    public static String getValueByKey(Integer key) {
        InTypeEnum inTypeEnum = classMap.get(key);

        if (ObjectUtil.isEmpty(inTypeEnum)) {
            return StringPool.EMPTY;
        }

        return inTypeEnum.getValue();
    }

}
