/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package com.fjhx.scheduling.controller;

import com.fjhx.entity.scheduling.Scheduling;
import com.fjhx.entity.scheduling.StockSchedulingTest;
import com.fjhx.scheduling.service.ISchedulingService;
import com.fjhx.scheduling.service.StockSchedulingTestService;
import com.fjhx.utils.HmacUtil;
import io.swagger.annotations.Api;
import lombok.AllArgsConstructor;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.StringUtil;
import org.springframework.web.bind.annotation.*;

import java.util.Date;

/**
 * 排班
 *
 * @author BladeX
 * @since 2022-07-27
 */
@RestController
@AllArgsConstructor
@RequestMapping("/hx/storage/open/api/scheduling")
@Api(value = "仓库", tags = "仓库接口")
public class SchedulingController extends BladeController {

    private final ISchedulingService schedulingService;
    private final StockSchedulingTestService stockSchedulingTestService;

    /**
     * 新增
     */
    @PostMapping("/save")
    public R save(@RequestBody Scheduling scheduling, @RequestHeader("Authority") String authority, @RequestHeader("date") String date) {
        if (!StringUtil.equals(HmacUtil.encrypt("fjhx" + date, "fjhx@2012"), authority)) {
            return R.fail("无权限");
        }
        scheduling.setCreateTime(new Date());
        scheduling.setId(null);
        return R.status(schedulingService.save(scheduling));
    }

    /**
     * 新增
     */
    @PostMapping("/test/save")
    public R testSave(@RequestBody StockSchedulingTest scheduling, @RequestHeader("Authority") String authority, @RequestHeader("date") String date) {
        if (!StringUtil.equals(HmacUtil.encrypt("fjhx" + date, "fjhx@2012"), authority)) {
            return R.fail("无权限");
        }
        scheduling.setCreateTime(new Date());
        scheduling.setId(null);
        return R.status(stockSchedulingTestService.save(scheduling));
    }

}
